# DFL IPC

Two very simple classes for IPC, namely DFL::IPC::Server and DFL::IPC::Client, especially for communication between two instances of the same application. These classes are used in DFL::Application.
Additionally, another class DFL::IPC::Wayfire is also provided to communicate with [Wayfire Compositor](https://github.com/WayfireWM/wayfire.git).

### Dependencies:
* <tt>Qt5 Core and Gui</tt>
* <tt>meson   (For configuring the project)</tt>
* <tt>ninja   (To build the project)</tt>


### Notes for compiling - linux:

- Install all the dependencies
- Download the sources
  * Git: `git clone https://gitlab.com/desktop-frameworks/ipc.git dfl-ipc`
- Enter the `dfl-ipc` folder
  * `cd dfl-ipc`
- Configure the project - we use meson for project management
  * `meson .build --prefix=/usr --buildtype=release`
  * The above command compiles the project using Qt6.
  * Pass `-Duse_qt_version=qt5` to the above command for Qt5 build.
- Compile and install - we use ninja
  * `ninja -C .build -k 0 -j $(nproc) && sudo ninja -C .build install`


### Known Bugs
* Please test and let us know


### Upcoming
* Support for sending arbitrary data (binary data) via IPC.
* Any feature that you ask for.


### Sample Usage
```cpp
DFL::IPC::Server server( "/tmp/server.socket", nullptr );
server.startServer();

connect(
    &server, &DFL::IPC::Server::messageReceived, [] ( QString msg, int fd ) {
        if ( msg == "Hello, world!" ) {
            server.reply( fd, "Hello!" );
        }

        else if ( msg.startsWith( "Bye-bye" ) ) {
            server.shutdown();
        }
    }
)

DFL::IPC::Client client( "/tmp/server.socket", nullptr );
client.connectToServer();

client.sendMessage( "Hello, world!" );
if ( client.waitForreply() && client.reply() != "Hello!" ) {
    client.sendMessage( "Bye-bye, Cruel world!" );
    client.disconnectFromServer();
}
```
